#! /usr/bin/env python

import argparse, requests, sys, time
requests.packages.urllib3.disable_warnings()

def run(args):
    redirections = 0
    timeouts = 0
    timeouts_after_redirection = 0
    url = "%s://%s:%u/%s" % (args.scheme, args.address, int(args.port), args.path)
    start = int(time.time())
    while int(time.time()) - start < int(args.timeout):
        try:
            resp = None
            resp = requests.head(url, allow_redirects=False, verify=False, timeout=(2.0, 5.0))
        except (requests.exceptions.Timeout, requests.exceptions.ConnectionError) as e:
            timeouts += 1
            if redirections > 0:
                timeouts_after_redirection += 1
            sys.stdout.write('!')
            sys.stdout.flush()
            time.sleep(2)
            continue
        except Exception as e:
            sys.stdout.write('### Timed out on %s, check networking. Exception follows:\n' % url)
            raise
        if resp and resp.status_code >= 200 and resp.status_code <= 299:
            sys.stdout.write('(Ready.)\n')
            sys.exit(0)
        else:
            redirections += 1
            sys.stdout.write('.')
            sys.stdout.flush()
            time.sleep(2)

    if timeouts_after_redirection:
        sys.stdout.write('\n(Connected but not ready at first, then no connection. Did network address change?)\n')
    elif timeouts:
        sys.stdout.write('\n(%s timed out, check networking.)\n' % url)
    elif redirections:
        sys.stdout.write('\n(%s redirected, assuming Not Ready.)\n' % url)
    else:
        sys.stdout.write('\n(%s gave weird response; timeout too short?)\n' % url)
    sys.exit(1)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Wait out AMC initialization')
    parser.add_argument("-p, --port", default="8443", help="Port at which to find AMC", dest="port")
    parser.add_argument("-s, --scheme", default="https", help="Scheme (protocol) for AMC", dest="scheme")
    parser.add_argument("-U, --username", default="local\\admin", help="AMC user name", dest='username')
    parser.add_argument("-P, --password", default="sonicwall", help="AMC user password", dest='password')
    parser.add_argument("-t, --timeout", default='2', help="Wait this many secs for connection", dest='timeout')
    parser.add_argument("address", default="192.168.0.10", help="Address at which to find AMC")
    parser.add_argument("path", default="Console/Help", help="Path to check")
    args = parser.parse_args()

    run(args)
